unit UsrComp;

interface

uses Windows, Sysutils, Messages, Classes, Controls, Graphics;

type
  TChangeEvent = procedure(Sender: TObject; var NewValue: Real) of object;
  TFormatEvent = procedure(Sender: TObject; var Caption: string) of object;

  TAngulo = class(TGraphicControl)
  private
    FValue: Real;
    FOnChange: TChangeEvent;
    FOnFormat: TFormatEvent;
    procedure Calcular(X, Y: Integer);
    procedure Dibujar;
    procedure SetValue(Value: Real);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Value: Real read FValue write SetValue;
    property Visible;
    property OnChange: TChangeEvent read FOnChange write FOnChange;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnFormat: TFormatEvent read FOnFormat write FOnFormat;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TAngulo]);
end;


{ TAngulo }

procedure TAngulo.Calcular(X, Y: Integer);
var
  a,b: Real;
  NewValue: Real;
begin
  a:= Y - (Height div 2);
  b:= (Width div 2) - X;
  if b = 0 then
  begin
    if a < 0 then
      NewValue:= pi/2
    else
      NewValue:= 3*pi/2;
  end else
  begin
    NewValue:= Arctan(a/b);
    if NewValue < 0 then
      NewValue:= NewValue+ (2*pi);
    if b > 0 then
      NewValue:= NewValue + pi;
    if NewValue > 2*pi then
      NewValue:= NewValue - (2*pi);
  end;
  Value:= NewValue;
end;

constructor TAngulo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width:= 100;
  Height:= 100;
  FValue:= 0;
end;

destructor TAngulo.Destroy;
begin
  inherited Destroy;
end;

procedure TAngulo.Dibujar;
var
  Bitmap: TBitmap;
  X, Y, W, H: Integer;
  Str: String;
begin
  Bitmap:= TBitmap.Create;
  try
    Bitmap.Canvas.Brush.Color:= Color;
    Bitmap.Width:= Width;
    Bitmap.Height:= Height;
    Bitmap.Canvas.Pen.Color:= clBlue;
    X:= Width div 2;
    Y:= Height div 2;
    W:= X;
    H:= Y;
    Bitmap.Canvas.MoveTo(X-W,Y);
    Bitmap.Canvas.LineTo(X+W,Y);
    Bitmap.Canvas.MoveTo(X,Y-H);
    Bitmap.Canvas.LineTo(X,Y+H);
    Bitmap.Canvas.MoveTo(X,Y);
    Bitmap.Canvas.Pen.Color:= clRed;
    Bitmap.Canvas.LineTo(
      Trunc(X + (W * Cos(FValue))), Trunc(Y - (H * Sin(FValue))));
    Bitmap.Canvas.Font:= Font;
    Str:= Format('%0.0f',[(Value * 180) / pi]);
    if Assigned(FOnFormat) then
      FOnFormat(Self,Str);
    if Length(Str) > 0 then
      with Bitmap.Canvas do
        TextOut(X - (TextWidth(Str) div 2),Y - (TextHeight(Str) div 2), Str);
    Canvas.Draw(0,0,Bitmap);
  finally
    Bitmap.Free;
  end;
end;

procedure TAngulo.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited;
  if ssLeft in Shift then
    Calcular(X,Y);
end;

procedure TAngulo.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if ssLeft in Shift then
    Calcular(X,Y);
end;

procedure TAngulo.Paint;
begin
  Dibujar;
end;

procedure TAngulo.SetValue(Value: Real);
begin
  if Assigned(FOnChange) then
    FOnChange(Self,Value);
  FValue := Value;
  Dibujar;
end;

end.
